/* 
-----------------------------------------------------------------------------
File: dba.js
Author: Karl Bishop (kfbishop@us.ibm.com)
-----------------------------------------------------------------------------
This is a custom Dojo module used in DojoDBA application
-----------------------------------------------------------------------------
(C) Copyright IBM Corp. 2010 All rights reserved.

This sample program is owned by International Business Machines Corporation or
one of its subsidiaries ("IBM") and is copyrighted and licensed, not sold.

The sample program is provided to you on an "AS IS" basis, without warranty of
any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER
EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE. Some jurisdictions do not allow for the exclusion or limitation of
implied warranties, so the above limitations or exclusions may not apply to you.
IBM shall not be liable for any damages you suffer as a result of using, modifying
or distributing the sample program or its derivatives.

Each copy of any portion of this sample program or any derivative work, must
include the above copyright notice and disclaimer of warranty.
----------------------------------------------------------------------------
*/
dojo.provide("dba");

dojo.require("dijit.Tree");
dojo.require("dojox.string.Builder");

//-- Remote REST URL's used by the application.  
//-- Parameters are replaced at runtime.
(function(){

dba = {
    
    urls : {
        DS:     "/resources/ds",
        SCHEMA: "/resources/ds/${ds}?showSysTables=${showSysTables}",
        DBINFO: "/resources/dbInfo/${ds}",
        SQL:    "/resources/ds/${ds}/sql",
        TABLE:  "/resources/ds/${ds}/tables/${table}"
    },

    //-- All of our data stores used within the application
    stores : {
        dataSources: null,
        schema: null,
        tableDetails: null
    },

    dataSource : null,
    busyMessage : [],

    //------------------------------------------------------------------------
    init : function() {
        // summary:
        //  Start DBA application
        var F = "dba.init(): ";
        console.debug(F, "Starting...");
        dba.loadDataSources();
        console.debug(F, "finished");
    },

    //------------------------------------------------------------------------
    loadDataSources : function() {
        // summary:
        //    Load iniital Data Source list
        var F = "dba.loadDataSources(): ";
        console.debug(F, "Starting...");
        dba.busy("Data Sources", true);
        try {
            dba.stores.dataSource = new dojo.data.ItemFileReadStore( {url: dba.urls.DS} );
            dba.stores.dataSource.fetch({
                sort: [{attribute:"uid"}],
                onComplete: function(items, request) {
                    try {
                        console.debug(F, "Data Sources loaded. #:", items.length);
                        var key = dba.stores.dataSource.getValue(items[0], "uid");
                        console.debug( F, "Setting DS default to: ", key );
        
                        var dsInput = dijit.byId("dataSource");
                        dsInput.attr({
                            store : dba.stores.dataSource,
                            value : key,
                            disabled : false
                        });
                        
                        var dsButton = dijit.byId("loadDataSourceButton");
                        dojo.connect(dsButton, "onClick", dba, "loadDatabaseInfo");
                        dojo.connect(dsButton, "onClick", dba, "loadDatabaseSchema");
                        dsButton.attr( "disabled", false );
                    } catch ( e2 ) {
                        dba.error(F,e2);
                    } finally {
                        dba.busy("Data Sources", false);
                        console.debug( F, "Finished" );
                    }
                }
            });
        } catch( e1 ) {
            dba.error(F, e1);
        }
    },

    //------------------------------------------------------------------------
    loadDatabaseInfo : function() {
        var F = "dba.loadDatabaseInfo(): ";
        dba.dataSource = dijit.byId("dataSource").getValue();
        console.debug(F, "Loading ", dba.dataSource);
    
        //-- Load up dbInfo too
        dba.busy("Database Info", true);
        dojo.byId("dbInfo_dbUrl").innerHTML = "Loading...";
        dojo.byId("dbInfo_dbName").innerHTML = "Loading...";
        dojo.byId("dbInfo_dbVersion").innerHTML = "Loading...";
        dojo.byId("dbInfo_driverName").innerHTML = "Loading...";
        dojo.xhrGet( {
            url: dojo.string.substitute( dba.urls.DBINFO, {ds:dba.dataSource} ),
            handleAs: "json",
            load: function(response, ioArgs) {
                console.debug(F, "Received DBInfo: ", response);
                dojo.byId("dbInfo_dbUrl").innerHTML = response.db_url;
                dojo.byId("dbInfo_dbName").innerHTML = response.db_name;
                dojo.byId("dbInfo_dbVersion").innerHTML = response.db_version;
                dojo.byId("dbInfo_driverName").innerHTML = response.driver_name;
            //dojo.byId("dbInfo_driverVersion").innerHTML = response.driver_version;
                dba.busy("Database Info", false);
            },
            error: function(response, ioArgs) {
                console.error(F,"load error: ", response, ioArgs);
                dba.busy("Database Info", false);
            }
        } );
    },
    
    //------------------------------------------------------------------------
    loadDatabaseSchema : function() {
        // summary:
        //        Load Schema Data Store for desired Data Source
        var F = "dba.loadDatabaseSchema(): ";
    
        dba.dataSource = dijit.byId("dataSource").attr("value");
        console.debug(F, "Loading: ", dba.dataSource);
    
        //-- Update title
        dojo.byId("dataSourceTitle").innerHTML = dba.dataSource;
    
        //-- Load up DataStore
        dba.busy("Database Schema", true);
        var showSys = dijit.byId("showSysTables").attr("value");
        console.debug("ShowSysTable = ", dijit.byId("showSysTables"));
        var url = dojo.string.substitute( dba.urls.SCHEMA, {ds:dba.dataSource, showSysTables: showSys} );
        dba.stores.schema = new dojo.data.ItemFileReadStore( {url:url} );
        dba.stores.schema.fetch({
            onComplete: function(items, request) {
                console.debug(F, "Schema store loaded: ", dba.stores.schema);
                //-- Create Schema Tree
                dba.createSchemaTree();
                //-- Advance accordion to schema tree pane    
                dijit.byId("accordion").forward();
                //-- Change value of editor and enable execute button
                dijit.byId("sqlEditor").attr("value",
                        "Enter SQL here, or select a table from the Schema tree to generate a sample query");
                dijit.byId("executeButton").attr("disabled", false);
                dba.busy("Database Schema", false);
            },
            onError: function(e) {
                console.error(F,e);
                dba.busy("Database Schema", false);
            }
        });
    },
    
    //------------------------------------------------------------------------
    createSchemaTree : function() {
        var F = "dba.createSchemaTree(): ";
        console.debug(F, "Starting");
        var schemaTree = dijit.byId("schemaTree");
        if ( schemaTree ) {
            console.debug(F, "Destroying existing schema tree");
            schemaTree.destroyRecursive();
        }
        var div = dojo.create("div", {}, dojo.byId("schemaTreeContainer"), "only" );
        schemaTree = new dijit.Tree({
            id       : 'schemaTree',
            store    : dba.stores.schema,
            labelAttr: 'label',
            typeAttr : "type",
            onClick  : dba.tableSelected
        }, div);
    },

    //------------------------------------------------------------------------
    tableSelected : function( item ) {
        var F = "dba.tableSelected(): ";
        var table = dba.stores.schema.getValue(item,"uid");
        console.log(F, "Table ", table, item);
        var type = dba.stores.schema.getValue(item, "type"); 
        if ( type === "table" ) {
            console.log(F, "Got a table, building Sample SQL and results table");
            dba.busy("Table Details", true);
            var url = dojo.string.substitute( dba.urls.TABLE, {ds:dba.dataSource, table:table} );
            dba.stores.tableDetails = new dojo.data.ItemFileReadStore( {url:url} );
            dba.stores.tableDetails.fetch({
                onComplete: function(items, request) {
                    console.debug(F, "Table details loaded for: ", table, ", data: ", dba.stores.tableDetails);
                    
                    //-- Populate editor with sample SQL for this table
                    //-- Use full editor based HTML formatting, just because.
                    console.debug(F,"Building editor query string for table: ", table);
                    var sql = "<b>SELECT</b><br /><blockquote>";
                    dojo.forEach(items, function(item, idx) {
                        sql += ( idx > 0 ) ? ", " : "";
                        sql += dba.stores.tableDetails.getValue(item, "name");
                    } );
                    sql += "<br /></blockquote><b>FROM</b><br /><blockquote>" + table + "</blockquote><br />";
                    dijit.byId("sqlEditor").attr("value", sql );
                    
                    //-- Put table name in the pane
                    dojo.byId('tableDetailsName').innerHTML = table;
                    
                    //-- Advance accordion to schema tree pane    
                    dijit.byId("accordion").forward();

                    var tableDetailsGrid = dijit.byId("tableDetailsGrid");
                    tableDetailsGrid.setStore(dba.stores.tableDetails);
                    tableDetailsGrid.startup();
                    
                    dba.busy("Table Details", false);
                },
                onError: function(e) {
                    dba.error(F,e);
                    dba.busy("Database Schema", false);
                }
            });
        } else {
            console.log(F, "Not a table (", type, ")");
            dojo.publish("toaster", [{message: "Select a table type object to view details"}] );
        }
    },

    //------------------------------------------------------------------------
    executeSql : function() {
        var F = "dba.executeSql(): ";
        console.debug(F, "Starting...");
        dba.busy("SQL Query", true);
    
        var sql = dijit.byId("sqlEditor").attr("value");
        //-- Editor uses a lot of HTML for formatting. We need to strip them for SQL string
        console.debug(F,"SQL = ", sql);
        sql = sql.replace(/<[^<>]*>/g, " ");
        console.debug(F,"SQL = ", sql);
        dojo.xhrPost( {
            url: dojo.string.substitute(dba.urls.SQL, {ds:dba.dataSource}),
            content: {
                "sql": sql
            },
            handleAs: "json",
            timeout: 60000,
            load: function(response, ioArgs) {
                dba.processSqlResults(sql, response);
                dba.busy("SQL Query", false);
            },
            error: function(e) {
                dba.error(F,e);
                dba.busy("SQL Query", false);
            }
        } );
    },

    //------------------------------------------------------------------------
    processSqlResults : function( sql, response ) {
        var F = "dba.processSqlResults(): ";
        console.debug(F, "Starting...");
    
        try {
            console.debug(F,"SQL: ", sql);
            console.debug(F,"Response: ", response);

            var resultsGrid = dijit.byId("resultsGrid");
            if ( resultsGrid ) {
                resultsGrid.destroyRecursive();
            }
            dojo.byId("resultsContainer").innerHTML = "";
            
            var div;
            if ( "Error" === response.messageType ) {
                dba.error(F,"Error processing SQL statement");
                div = dojo.create("div",{"class":"error", "innerHTML":response.message}, "resultsContainer");
            } else if ( "Info" === response.messageType ) {
                console.info(F,"Response message: "+response.message);
                if( response.content && response.content.items.length > 0 ) {
                    // Have a table, so show a toaster message
                    dojo.publish("toaster", [{message: response.message}] );

                    var resultsContentStore = new dojo.data.ItemFileReadStore( {data:response.content} );

                    //-- Build layout from the headers
                    var layout = [];
                    dojo.forEach( response.headers, function(hdr) {
                        var cell = {name:hdr.name, field:hdr.field, width:"auto"};
                        if ( "number" === hdr.type) {
                            cell.cellStyles = "text-align: right;";
                        } else if ( "boolean" === hdr.type ) {
                            cell.cellStyles = "text-align: center";
                        }
                        layout.push( cell );
                    });
                    console.debug(F,"ResultsGridLayout: ", layout);

                    // create a new grid:
                    resultsGrid = new dojox.grid.DataGrid({
                        id: "resultsGrid",
                        store: resultsContentStore,
                        query: { "C1": '*' },
                        columnReordering: true,
                        clientSort: true,
                        rowSelector: '0px',
                        selectionMode: 'single',
                        structure: layout,
                        height: "100%",
                        noDataMessage: "No records found..."
                    }, dojo.create("div",{},"resultsContainer") );
                    resultsGrid.startup();
                } else {
                    // no table, so just show the message
                    div = dojo.create("div",{"class":'title', "innerHTML":response.message}, "resultsContainer");
                }
            }
        } catch(e) {
            dba.error(F,e);
        } finally {
            dba.busy("SQL Query", false);
        }
    },

    //------------------------------------------------------------------------
    busy : function( /*string*/topic, /*bool*/start ) {
        var F = "dba.busy(): ";
        console.debug(F, "Topic=",topic, ", Start=", start );
        
        if( start ) {
            // Add item to list
            dba.busyMessage.push(topic);
        } else {
            // Remove from list
            dba.busyMessage.splice( dba.busyMessage.indexOf(topic), 1 );
        }
        
        if ( dba.busyMessage.length > 0 ) {
            dojo.byId('busyBar').style.visibility='visible';
            dojo.fadeIn({ node: 'busyBar', duration:300 }).play();
            dijit.byId('busy').update({ maximum: 1, progress: 1 } );
        } else {
            dojo.fadeOut({ node: 'busyBar', duration:800,
                onEnd: function(){
                    // remove progress bar from tab order
                    dojo.byId('busyBar').style.visibility='hidden';
                }
            }).play();
        }
    },
    
    //------------------------------------------------------------------------
    busyReport : function( percent ) {
           return "Processing: " + dba.busyMessage.join(", ");
    },
    
    //------------------------------------------------------------------------
    error : function( func, msg ) {
        console.error(func, msg);
        dojo.publish("toaster", [{
            message: (msg instanceof Error ? "Error: "+msg.message : msg) + "<br/>Reported by: " + func,
            type:"error"}] );
    },
    
    //------------------------------------------------------------------------
    toggleTheme : function() {
           dojo.toggleClass(dojo.body(), "soria");
           dojo.toggleClass(dojo.body(), "tundra");
           console.info("Theme set to: ", dojo.body().className);
    }

};

}() );
