/**
 * onList: GET (Collection)
 *
 * @success 200 Returns a list of all cars
 * @failure 503 systems down for maintenance
 * @format application/json, text/xml
 * @example
 * [
 *   {
 *     id: id,
 *     make: String,
 *     model: String,
 *     year: YYYY,
 *     number: XX,
 *     driver: {
 *       // Driver object
 *     },
 *     team: {
 *       // Team object
 *     }
 *   },
 *   ...
 * ]
 */
def onList() {
  logger.INFO {"onList starting"}
  // Extract all cars from storage and return
  def content = invokeMethod('rest/get', 'json', '/public/data/cars.json')
  invokeMethod('rest/send', 'json', content)
  logger.INFO {'onList finished'}
}

/**
 * onRetrieve: GET (Singleton), with possible binding extensions.
 *
 * @success 200 Returns car specific data for a given ID
 * @failure 503 systems down for maintenance
 * @format application/json
 * @example
 * {
 *     id: id,
 *     make: String,
 *     model: String,
 *     year: YYYY,
 *     number: XX,
 *     driver: {
 *       // Driver object
 *     },
 *     team: {
 *       // Team object
 *     }
 * }
 * 
 */
def onRetrieve() {
  // Get the ID of car to get, this param is fixed
  def carId = request.params.carId[]
  logger.INFO {"onRetrieve starting"}
  // Extract all cars from storage and return
  def content = invokeMethod('rest/get', 'json', "/public/data/car-${carId}.json")
  def formats = invokeMethod('rest/headers', 'getRequestHeaderValues', "Accept")
  if ( formats.contains("text/xml") ) {
    invokeMethod('rest/send', 'xml', content, "car")
  } else {
    invokeMethod('rest/send', 'json', content)
  }
  logger.INFO {'onRetrieve finished'}
}

/**
 * onCreate: POST (add)
 *
 * @success 201 Car object created
 * @failure 503 systems down for maintenance
 * @format application/json
 * @example
 * {
 *     make: String,
 *     model: String,
 *     year: YYYY,
 *     number: XX,
 *     driver: {
 *       // Driver object
 *     },
 *     team: {
 *       // Team object
 *     }
 * }
 * 
 */
def onCreate() {
  // The new data for the record with could be in any param
  def carData = request.params.carData[]
  logger.INFO {"Create starting"}
  // Insert a new car into storage.
  // Need to return the ID of the new car
}

 /**
  * onUpdate: PUT
  *
  * @success 201 Car object created
  * @failure 503 systems down for maintenance
  * @format application/json
  * @example
  * {
  *     id: id,
  *     make: String,
  *     model: String,
  *     year: YYYY,
  *     number: XX,
  *     driver: {
  *       // Driver object
  *     },
  *     team: {
  *       // Team object
  *     }
  * }
  * 
  */
def onUpdate() {
  // Get the ID of car to update, this param is fixed
  def carId = request.params.carId[]
  logger.INFO {"Update starting for: ${carId}"}
  // The data to update the record with could be in any param
  def carData = request.params.someData[]
  // Update data for existing car
}

  /**
   * onDelete: DELETE
   *
   * @success 200 Currently Unused
   * @failure 405 HTTP_BAD_METHOD: Deletes not allowed.
   * 
   */
def onDelete() {
  // Get the ID of car to delete, this param is fixed
  def carId = request.params.carId[]
  logger.INFO {"Delete starting for: ${carId}"}
  // Lets not allow deletion of Cars
  request.status = HttpURLConnection.HTTP_BAD_METHOD
}
